package org.warp.jcwdb;

import java.util.Map;
import java.util.Map.Entry;
import java.util.HashMap;

public class TypesManager {
	private final Map<Integer, DBTypeParser<?>> types;
	private final Map<Class<?>, DBTypeParser<?>> typesByClass;
	private DBTypeParser<?> fallbackParser;

	public TypesManager(JCWDatabase db) {
		types = new HashMap<>();
		typesByClass = new HashMap<>();
		DBStandardTypes.registerStandardTypes(db, this);
	}
	
	public <T> void registerType(Class<T> clazz, int type, DBTypeParser<T> parser) {
		this.types.put(type, parser);
		this.typesByClass.put(clazz, parser);
	}

	public void registerTypeFallback(DBTypeParser<?> parser) {
		this.fallbackParser = parser;
	}

	public <T> DBTypeParser<T> get(int type) {
		if (types.containsKey(type) == false) {
			if (fallbackParser == null) {
				throw new NoParserFoundException("The type " + type + " can't be parsed.");
			} else {
				return fallbackParser.cast();
			}
		}
		return types.get(type).cast();
	}

	public <T> DBTypeParser<T> get(Class<T> type) {
		if (typesByClass.containsKey(type) == false) {
			if (fallbackParser == null) {
				throw new NoParserFoundException("The class " + type.getSimpleName() + " can't be parsed.");
			} else {
				return fallbackParser.cast();
			}
		}
		return typesByClass.get(type).cast();
	}
}
